/* 
 * echo_mod.c
 *
 * echo effect module
 *
 * Copyright (C) 1998 Hermann Samso
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ncurses.h>
#include "types.h"
#include "delfin.h"
#include "display.h"
#include "wave.h"
#include "module.h"
#include "audiodrv.h"

typedef struct ECHO {
	BOOL active;
	INT number;
	ULONG offset;
	struct SAMPLE vol;
} ECHO;

ECHO echoes[MAXWAVES];
SAMPLE* echo_buffer;


void Echo(WAVE* wave, ECHO echo);

void EchoInit(void) {
	if (trace_on)
		fprintf(stderr, "EchoInit()\n");


	echo_buffer = (SAMPLE*)calloc(AudioGetBufferSize(), sizeof(SAMPLE));
	ModuleDisplay();
	
}


void EchoExit(void) {
	if (trace_on)
		fprintf(stderr, "EchoExit()\n");
}


void EchoCtrl(INT input) {
	INT idx = WaveGetIndex();
	INT keys[] = {
		'a',		// activate echo toggle
		'+',		// add echo
		'-', 		// sub echo
 		'7',		// add left volume
		'1',		// sub left volume
		'8',		// add right volume
		'2',		// sub right volume
		'y',		// sub offset
		'x',
		'c',		
		'n',		// add offset
		'b',
		'v'
	};


	if (trace_on)
		fprintf(stderr, "EchoCtrl()\n");
		
	if (input == keys[0]) {
		if (echoes[idx].active)
			echoes[idx].active = FALSE;
		else
			echoes[idx].active = TRUE;
	}
	if (input == keys[1])
		echoes[idx].number++;
	else if (input == keys[2])
		echoes[idx].number--;
	else if (input == keys[3])
		echoes[idx].vol.left = echoes[idx].vol.left + 0.1;
	else if (input == keys[4])
		echoes[idx].vol.left = echoes[idx].vol.left - 0.1;
	else if (input == keys[5])
		echoes[idx].vol.right = echoes[idx].vol.right + 0.1;
	else if (input == keys[6])
		echoes[idx].vol.right = echoes[idx].vol.right - 0.1;
	else if (input == keys[7])
		echoes[idx].offset = echoes[idx].offset - 100;
	else if (input == keys[8])
		echoes[idx].offset = echoes[idx].offset - 10;
	else if (input == keys[9])
		echoes[idx].offset = echoes[idx].offset - 1;
	else if (input == keys[10])
		echoes[idx].offset = echoes[idx].offset + 100;
	else if (input == keys[11])
		echoes[idx].offset = echoes[idx].offset + 10;
	else if (input == keys[12])
		echoes[idx].offset = echoes[idx].offset + 1;
	else
		usleep(200);
		
	ModuleDisplay();
}

void EchoExec(void) {
	INT i;
	INT nr_waves = WaveHowMany();
	WAVE* wave;
	
	if (trace_on)
		fprintf(stderr, "EchoExec()\n");

	for (i = 0; i < nr_waves; i++) {
		wave = WaveGetByIndex(i);
		if (wave->active && echoes[i].active)
			Echo(wave, echoes[i]);
	}
}



void EchoDisplay(void) {
	INT i = WaveGetIndex();

	if (trace_on)
		fprintf(stderr, "EchoDisplay()\n");

	DisplayClearBottom();
	mvprintw(LINES-2, 0, "[%s]", echo_mod.name);
	mvprintw(LINES-2, 8, "lvol:%f", echoes[i].vol.left);
	mvprintw(LINES-1, 8, "rvol:%f", echoes[i].vol.right);
	mvprintw(LINES-1, 0, "active:%d", echoes[i].active);
	mvprintw(LINES-2, 23, "number:%d", echoes[i].number);
	mvprintw(LINES-1, 23, "offset:%li", echoes[i].offset);
}


void Echo(WAVE* wave, ECHO echo) {
	INT i, j;
	ULONG buf_size = AudioGetBufferSize();
	ULONG pos;
	FLOAT lvol, rvol;
			

	if (trace_on)
		fprintf(stderr, "Echo(%s)\n", wave->name);


	if (wave == NULL)
		return;

	lvol = echo.vol.left;
	rvol = echo.vol.right;
	for (i = 0; i < echo.number; i++) {
		pos = (i+1) * echo.offset + wave->position;
		if (pos >= 0 && pos < wave->size) {
			for (j = 0; j < buf_size && pos < wave->size; j++, pos++) {
				wave->buffer[j].left = 	(wave->buffer[j].left + 
										wave->samples[pos].left * lvol) / 2;
				wave->buffer[j].right = (wave->buffer[j].right +
										wave->samples[pos].right * rvol) / 2;
			}		
		}
		lvol = lvol * echo.vol.left;
		rvol = rvol * echo.vol.right;
	}
	
}

		
MODULE echo_mod = {
	NULL,			/* next MODULE  in list */
	NULL,			/* previous MODULE in list */
	"echo",
	"v1998.2.14 by Hermann Samso",
	EchoInit,
	EchoExit,
	EchoCtrl,
	EchoExec,
	EchoDisplay
};

